package asclass
{
	import asclass.events.LEDBoardEvent;
	import asclass.events.LEDColorEvent;
	
	import flash.display.Graphics;
	import flash.events.EventDispatcher;
	import flash.events.MouseEvent;
	
	import flashx.textLayout.formats.TextAlign;
	
	import mx.events.FlexEvent;
	import mx.graphics.SolidColor;
	import mx.graphics.SolidColorStroke;
	
	import spark.components.Group;
	import spark.components.Label;
	import spark.primitives.Ellipse;
	import spark.primitives.Rect;

	[Event(name="new_selection", type="asclass.events.LEDBoardEvent")]
	
	public class LEDBoard extends Group
	{
		private const DEFAULT_WIDTH:int = 100;
		private const DEFAULT_HEIGHT:int = 100;
		
		public var commandDispatcher:LEDCommandDispatcher = null;
		private var _clrNavigator:EventDispatcher = null;
		

		
		private var _redValue:uint = 0;
		private var _blueValue:uint = 0;
		private var _greenValue:uint = 0;
		private var _address:int = 255;
		private var _currentColor:uint;
		private var _isActive:Boolean;
		private var _title:String = "";
		private var titleBox:Label;
		private var addressBox:Label;
		private var _selected:Boolean = false;
		
		private var newlySelected:Boolean = false;
		private var circle:Ellipse;
		private var circleFill:SolidColor;
		private var circleStroke:SolidColorStroke;
		
		private var box:Rect;
		private var boxStroke:SolidColorStroke;
		
		
		public function LEDBoard(pAddress:int = 255, pRed:int = 0, pGreen:int = 0, pBlue:int = 0)
		{
			super();
			
			_address = pAddress;
			_redValue = pRed;
			_blueValue = pBlue;
			_greenValue = pGreen;
			
		
			this.width = DEFAULT_WIDTH;
			this.height = DEFAULT_HEIGHT;
			
			boxStroke = new SolidColorStroke(0x00FF00,2);
			box = new Rect();
			box.width = DEFAULT_WIDTH-40;
			box.height = DEFAULT_HEIGHT-40;
			box.x = 20;
			box.y = 20;
			box.stroke = boxStroke;
			
			this.addElement(box);
			
			
			circleFill = new SolidColor(0);
			circleStroke = new SolidColorStroke(0,2);
			circle = new Ellipse();
			circle.width = (DEFAULT_WIDTH) - (52);
			circle.height = (DEFAULT_WIDTH) - (52);
			circle.x = (DEFAULT_WIDTH - circle.width) /2;
			circle.y = (DEFAULT_HEIGHT - circle.height) /2;
			circle.fill = circleFill;
			circle.stroke = circleStroke;
			
			this.addElement(circle);
			
			titleBox = new Label();
			titleBox.width = DEFAULT_WIDTH - 10;
			titleBox.x = 5;
			titleBox.y = 5;
			titleBox.setStyle("textAlign",TextAlign.CENTER);
			titleBox.text = title;
			this.addElement(titleBox);
			
			addressBox = new Label();
			addressBox.width = DEFAULT_WIDTH - 10;
			addressBox.x = 5;
			addressBox.y = DEFAULT_HEIGHT - 12;
			addressBox.setStyle("textAlign",TextAlign.CENTER);

			updateAddressBox();
			this.buttonMode = true;
			this.addElement(addressBox);

			this.addEventListener(MouseEvent.CLICK,handleClick);
			this.addEventListener(FlexEvent.CREATION_COMPLETE, creationCompleteHandler);
			
			
			
		}
		public function unRegister():void
		{
			this.removeEventListener(MouseEvent.CLICK,handleClick);
			this.removeEventListener(FlexEvent.CREATION_COMPLETE, creationCompleteHandler);	
			_clrNavigator.removeEventListener(LEDColorEvent.LED_COLOR_EVENT,recieveColorEvent);
		}

		public function get clrNavigator():EventDispatcher
		{
			return _clrNavigator;
		}

		public function set clrNavigator(value:EventDispatcher):void
		{
			_clrNavigator = value;
			value.addEventListener(LEDColorEvent.LED_COLOR_EVENT,recieveColorEvent);
		}

		private function recieveColorEvent(event:LEDColorEvent):void
		{
			if (this.selected)
			{
				if (this.newlySelected)
				{
					this.redValue = event.redValue;
					this.blueValue = event.blueValue;
					this.greenValue = event.greenValue;
					this.newlySelected = false;
				} else
				{
					switch (event.colorCode)
					{
						case LEDCommandDispatcher.SET_RED_CODE:
							this.redValue = event.colorValue;
							break;
						case LEDCommandDispatcher.SET_BLUE_CODE:
							this.blueValue = event.colorValue;
							break;
						case LEDCommandDispatcher.SET_GREEN_CODE:
							this.greenValue = event.colorValue;
							break;
					}
				}

			}
		}
		
		public function get selected():Boolean
		{
			return _selected;
		}

		public function set selected(value:Boolean):void
		{
			_selected = value;
			if (value)
			{
				newlySelected = true;
				
			}
			updateBackground();
			var anEvent:LEDBoardEvent = new LEDBoardEvent(LEDBoardEvent.NEW_SELECTION);
			anEvent.blueValue = this.blueValue;
			anEvent.redValue = this.redValue;
			anEvent.greenValue = this.greenValue;
			this.dispatchEvent(anEvent);
		}
		private function creationCompleteHandler(event:FlexEvent):void
		{
			updateBackground();
		}
		private function handleClick(event:MouseEvent):void
		{
			this.selected = !this.selected;
		}
		private function updateBackground():void
		{
			/*
			this.graphics.clear();
			this.graphics.beginFill(0xFFFFFF);
			*/
			if (this.selected)
			{
			
				boxStroke.color = 0x00FF00;
			} else
			{
				
				boxStroke.color = 0xCCCCCC;
			}
			/*
			this.graphics.drawRect(20,20,DEFAULT_WIDTH-40,DEFAULT_HEIGHT-40);
			this.graphics.endFill();
			*/

			updateLEDColor();
		}

		private function updateAddressBox():void
		{
			addressBox.text = "Address: " + this.address;
		}
			
		public function get title():String
		{
			return _title;
		}

		public function set title(value:String):void
		{
			titleBox.text = value;
			_title = value;
		}

		private function updateLEDColor():void
		{
			/*
			this.graphics.beginFill(this.currentColor);
			this.graphics.lineStyle(2,0);
			this.graphics.drawCircle(DEFAULT_WIDTH/2,DEFAULT_HEIGHT/2,(DEFAULT_WIDTH/2) - (52/2));
			this.graphics.endFill();
			*/
			circleFill.color = this.currentColor;
		}

		public function get isActive():Boolean
		{
			return _isActive;
		}

		public function set isActive(value:Boolean):void
		{

			_isActive = value;
		}

		public function get currentColor():uint
		{
			return (_redValue << 16) | (_greenValue << 8) | (_blueValue);
		}

		public function set currentColor(value:uint):void
		{
			//_currentColor = value;
			updateLEDColor();
		}

		public function get address():int
		{
			return _address;
		}

		public function set address(value:int):void
		{
			_address = value;
			updateAddressBox();
		}

		public function get greenValue():uint
		{
			return _greenValue;
		}

		public function set greenValue(value:uint):void
		{
			_greenValue = value;
			commandDispatcher.setLEDColor(this.address,LEDCommandDispatcher.SET_GREEN_CODE,value);
			currentColor = 0; //Trigger changes based on binding
		}

		public function get blueValue():uint
		{
			return _blueValue;
		}

		public function set blueValue(value:uint):void
		{
			_blueValue = value;
			commandDispatcher.setLEDColor(this.address,LEDCommandDispatcher.SET_BLUE_CODE,value);
			currentColor = 0; //Trigger changes based on binding
		}

		public function get redValue():uint
		{
			return _redValue;
		}

		public function set redValue(value:uint):void
		{
			_redValue = value;
			commandDispatcher.setLEDColor(this.address,LEDCommandDispatcher.SET_RED_CODE,value);
			currentColor = 0; //Trigger changes based on binding
		}

	}
}