package asclass
{
	import asclass.events.LEDCommandEvent;
	
	import flash.events.ProgressEvent;
	import flash.events.TimerEvent;
	import flash.net.Socket;
	import flash.utils.ByteArray;
	import flash.utils.Timer;
	
	[Event(name="ledtableavailable", type="asclass.events.LEDCommandEvent")]
	
	[Bindable]
	public class LEDCommandDispatcher extends Socket
	{
		private var _numberOfLEDs:Number;
		private var _mainPowerState:Boolean;
		public var ledStateResponse:Array = new Array(); 
		
		private var powerRequestTimer:Timer = new Timer(500);
		
		public var voltage:Number = 0;
		public var current:Number = 0;
		
		
		public static const SET_POWER_CODE:int = 1;
		public static const SET_RED_CODE:int = 2;
		public static const SET_BLUE_CODE:int = 4;
		public static const SET_GREEN_CODE:int = 3;
		public static const SET_ALL_CODE:int = 5;
		public static const REQUEST_LED_TABLE_CODE:int = 10;
		public static const SYSTEM_RESET_CODE:int = 13;
		public static const ADD_ADDRESS_CODE:int = 21;
		public static const DELETE_ADDRESS_CODE:int = 22;
		public static const SET_RED_SCALE:int = 32
		public static const SET_RED_SCALE_CONFIRM:int = 42
		public static const SET_GREEN_SCALE:int = 33
		public static const SET_GREEN_SCALE_CONFIRM:int = 43
		public static const SET_BLUE_SCALE:int = 34
		public static const SET_BLUE_SCALE_CONFIRM:int = 44
		
		public function get numberOfLEDs():Number
		{
			return _numberOfLEDs;
		}
		public function get mainPowerState():Boolean
		{
			return _mainPowerState;
		}
		public function set mainPowerState(val:Boolean):void
		{
			
			_mainPowerState = val;
			
		}
		
		public function LEDCommandDispatcher(host:String=null, port:int=0)
		{
			super(host, port);
			this.addEventListener(ProgressEvent.SOCKET_DATA, dataRecieved);
			powerRequestTimer.stop();
			powerRequestTimer.addEventListener(TimerEvent.TIMER,powerRequestTimerEvent);
		}
		public function fetchLEDTable():void
		{
			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(LEDCommandDispatcher.REQUEST_LED_TABLE_CODE); //command code
			bArray.writeByte(110); //filler
			bArray.writeByte(210); //filler
			this.writeBytes(bArray);
			this.flush();
		}
		public function get powerUpdatesEnabled():Boolean
		{
			return this.powerRequestTimer.running;
		}
		public function set powerUpdatesEnabled(value:Boolean):void
		{
			if (value == true)
			{
				powerRequestTimer.start();
			}
			else
			{
				powerRequestTimer.stop();
				voltage = 0;
				current = 0;
			}
		}
		private function powerRequestTimerEvent(event:TimerEvent):void
		{
			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(16); //command code
			bArray.writeByte(0); //filler
			bArray.writeByte(0); //filler
			
			this.writeBytes(bArray);
			this.flush();
			
			
		}
		private function dataRecieved(event:ProgressEvent):void
		{
			if (this.bytesAvailable > 0)
			{
				var responseCode:Number = this.readByte(); //response code indicates which data we are recieving
				if (responseCode == 10) //request for LEDTable
				{
					//First byte is power state
					mainPowerState = Boolean(this.readByte());
					//Second byte is number of LED Boards
					_numberOfLEDs = this.readByte();
					//The rest are the states of the individual boards
					ledStateResponse = new Array(); 
					for (var i:int = 0; i < _numberOfLEDs; i++)
					{
						var ledState:LEDState = new LEDState();
						ledState.address = this.readUnsignedByte();
						ledState.red = this.readUnsignedByte();
						ledState.green = this.readUnsignedByte();
						ledState.blue = this.readUnsignedByte();
						ledStateResponse.push(ledState);
					}
					var anEvent:LEDCommandEvent = new LEDCommandEvent(LEDCommandEvent.LED_TABLE_AVAILABLE);
					this.dispatchEvent(anEvent);
				}
				if (responseCode == 16) //request for Voltage
				{
					this.voltage = this.readUnsignedShort() / 100;
					this.current = this.readUnsignedShort() / 1000;
				}
				
			}
		}
		public function disconnectMe():void
		{
			
			if (this.connected)
			{
				var bArray:ByteArray = new ByteArray();
				bArray.writeByte(7); //command
				bArray.writeByte(17);
				bArray.writeByte(27);
				this.writeBytes(bArray);
				this.flush();
			}
		}
		public function setLEDColor(address:Number, colorCode:Number, intensity:Number):void
		{
			if (this.connected)
			{
				var bArray:ByteArray = new ByteArray();
				bArray.writeByte(colorCode); //command
				bArray.writeByte(address); //address
				bArray.writeByte(intensity);
				this.writeBytes(bArray);
				this.flush();
				
			}
		}
		public function setPowerState(powerState:Boolean):void
		{
			if (this.connected)
			{
				var bArray:ByteArray = new ByteArray();
				bArray.writeByte(SET_POWER_CODE); //command	code
				bArray.writeByte(int(powerState));
				bArray.writeByte(11); //filler
				this.writeBytes(bArray);
				this.flush();
				mainPowerState = powerState;
			}
		}
		
		public function addLEDAddress(address:uint):void
		{
			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(ADD_ADDRESS_CODE); //command	code
			bArray.writeByte(address);
			bArray.writeByte(210); //filler
			this.writeBytes(bArray);
			this.flush();
		}
		public function deleteLEDAddress(address:uint):void
		{
			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(DELETE_ADDRESS_CODE); //command	code
			bArray.writeByte(address);
			bArray.writeByte(220); //filler
			this.writeBytes(bArray);
			this.flush();
		}
		public function sendControllerReset():void
		{
			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(SYSTEM_RESET_CODE); //command	code
			bArray.writeByte(130); //verification bytes
			bArray.writeByte(230); 
			this.writeBytes(bArray);
			this.flush();
		}
		public function setColotScale(address:Number, colorCode:Number, scale:Number):void
		{
			var command:int = 0;
			var commandConfirm:int = 0;
			if (colorCode == SET_RED_CODE)
			{
				command = SET_RED_SCALE;
				commandConfirm = SET_RED_SCALE_CONFIRM;
			}
			if (colorCode == SET_GREEN_CODE)
			{
				command = SET_GREEN_SCALE;
				commandConfirm = SET_GREEN_SCALE_CONFIRM;
			}
			if (colorCode == SET_BLUE_CODE)
			{
				command = SET_BLUE_SCALE;
				commandConfirm = SET_BLUE_SCALE_CONFIRM;
			}

			var bArray:ByteArray = new ByteArray();
			bArray.writeByte(command); 
			bArray.writeByte(address); 
			bArray.writeByte(scale); 
			bArray.writeByte(commandConfirm); 
			bArray.writeByte(address); 
			bArray.writeByte(scale); 
			this.writeBytes(bArray);
			this.flush();
		}
		
	}
}